package Testrig 
  
model Wegvorgabe 
  Modelica.Blocks.Interfaces.RealInput u;                          // Laden des Modelica Konnektors RealInput
  Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a;   // Laden des Modelica Konnektors Flange_a
equation 
  flange_a.s = u;
end Wegvorgabe;
  
model Luftfederdaempfer 
  
  parameter Real Ti(unit="K")=293;
  parameter Real pi(unit="Pa")=2070000;
  parameter Real k(unit="W/K/m2")=150;
  parameter Real R(unit="J/kg/K")=287.2;
  parameter Real gamma=1.4;
  parameter Real cp(unit="J/kg/K")=1005;
  parameter Real V1(unit="m3")=0.000752;
  parameter Real A1(unit="m2")=0.0081;
  parameter Real Aw1(unit="m2")=0.2;
  parameter Real V2(unit="m3")=0.00126;
  parameter Real A2(unit="m2")=0.0118;
  parameter Real Aw2(unit="m2")=0.1;
  parameter Real alpha=0.58;
  parameter Real Ab(unit="m2")=40e-6;
  Real rho1(start=pi/(R*293),unit="kg/m3");
  Real T1(unit="K");
  Real p1(start=pi,unit="Pa");
  Real T2(start=293,unit="K");
  Real rho2(start=pi/(R*293),unit="kg/m3");
  Real p2(start=pi,unit="Pa");
  Real Me;
  Real ae(unit="m/s");
  Real rhoe(unit="kg/m3");
  Real mflow(unit="kg/s");
  Real F;
  Real z(unit="m");
  Real zdot;
  Real fvonMe;
  Real T_max;
  Real rho_max;
  Real druckv "Druckverhltnis";
    
  Modelica.Mechanics.Translational.Interfaces.Flange_a flange_a;  // Laden des Modelica Konnektors Flange_a
  Modelica.Mechanics.Translational.Interfaces.Flange_b flange_b;  // Laden des Modelica Konnektors Flange_b
  
equation

  z = flange_a.s-flange_b.s;
  zdot=der(z);

  flange_a.f = F;
  flange_b.f = -F;
  
  der(rho1) = (mflow*tanh(1000*(p2-p1)/p2) - rho1*zdot*A1)/(V1 + z*A1);
  der(rho2) = (-mflow*tanh(1000*(p2-p1)/p2) + rho2*zdot*A2)/(V2 - z*A2);
  der(p1) =( mflow*tanh(1000*(p2-p1)/p2)*cp*T_max - gamma/(gamma - 1)*p1*A1*zdot - k*Aw1*(T1 - Ti))/((V1 + z*A1)/(gamma - 1));
  der(p2) = (-mflow*tanh(1000*(p2-p1)/p2)*cp*T_max + gamma/(gamma - 1)*p2*A2*zdot - k*Aw2*(T2 - Ti))/((V2 - z*A2)/(gamma - 1));
  
  T_max = if p1>p2 then T1 else T2;
  rho_max = if p1>p2 then rho1 else rho2;
  
  p1=rho1*R*T1;
  p2=rho2*R*T2;
  
  druckv = min(p1,p2)/max(p1,p2); // Berechnung des Druckverhltnis
  
  rhoe = if druckv > 0.528 then rho_max*((gamma-1)/2*Me^2+1)^(-1/(gamma-1)) else 0.634*rho_max;
  ae = if druckv > 0.528 then (gamma*R*T_max)^0.5*((gamma-1)/2*Me^2+1)^(-0.5) else 0.913*(gamma*R*T_max)^0.5;
  fvonMe = (1/druckv)^((gamma-1)/gamma);
  Me = if druckv >= 0.528 and fvonMe >= 1 then (2*(fvonMe - 1)/(gamma - 1))^0.5 else if druckv < 0.528 and fvonMe >= 1 then 1 else 0;
  
  mflow=alpha*Ab*ae*Me*rhoe;

  F=p2*A2 - p1*A1 - 1e5*(A2 - A1);

end Luftfederdaempfer;

 
model Systemaufbau          // In diesem Modell wird das gesamte System zusammengesetzt     
    Wegvorgabe Wegvorgabe1;   // Einfgen des Modells Wegvorgabe. 
    Luftfederdaempfer Luftfederdaempfer1;     // Hier das Federbein einfgen, analog wie bei Block Wegvorgabe
    Modelica.Mechanics.Translational.Fixed Fixed1;                // Einfgen des Modelica Blocks Fixed.
    Modelica.Blocks.Sources.Sine Sine1(amplitude=0.01, freqHz=0.01); // Einfgen des Modelica Blocks Sine
  equation 
    connect(Sine1.y,Wegvorgabe1.u);       // Hier wird der Block Sine1 mit dem Block Wegvorgabe1 verbunden
    connect(Wegvorgabe1.flange_a, Luftfederdaempfer1.flange_b);  // Hier den Block Wegvorgabe mit Federbein verbinden
    connect(Luftfederdaempfer1.flange_a ,Fixed1.flange_b);       // Hier den Block Federbein mit Fixed verbinden
  end Systemaufbau;
  
end Testrig;
